// Copyright (2003) Michael L Perry.
// Permission is granted to use this source code without restriction
// privided the copyright notice appears without modification.

// FacetDemo.cpp
//
// Consumer of PersonnelHandler.

#include "PersonnelHandler.h"

#include <iostream>
#include <xercesc/sax/HandlerBase.hpp>

std::string findFileName( const char *relativePath )
{
   char absPath[512];
   if ( _fullpath( absPath, relativePath, sizeof(absPath)/sizeof(absPath[0]) ) == NULL )
      throw std::exception( "Failed to get full path." );

   return absPath;
}

std::string nameOf( const mapIDToPerson &people, const std::string &id )
{
   if ( id.empty() )
      return std::string();

   mapIDToPerson::const_iterator it = people.find( id );
   if ( it == people.end() )
      throw std::exception( "Person id not found" );

   const Person &p = it->second;
   return p.familyName + ", " + p.givenName;
}

int main(int argc, char* argv[])
{
   // Initialize the Xerces parser.
   try
   {
      XMLPlatformUtils::Initialize();
   }
   catch (const XMLException& toCatch) {
      char* message = XMLString::transcode(toCatch.getMessage());
      std::cout << "Error during initialization! :\n"
           << message << "\n";
      XMLString::release(&message);
      return 1;
   }

   try
   {
      // Parse the personnel XML file into a map.
      mapIDToPerson people;
      PersonnelHandler ph( people );
      ph.parse( findFileName( "personnel.xml" ).c_str() );

      // Display the information.
      std::cout << "People: " << people.size() << std::endl;
      for (
         mapIDToPerson::iterator itPerson = people.begin();
         itPerson != people.end();
         ++itPerson )
      {
         const Person &p = itPerson->second;
         std::cout << "  " << p.familyName << ", " << p.givenName << std::endl;
         std::cout << "    Email: " << p.email << std::endl;
         std::cout << "    Work web: " << p.workWeb << std::endl;
         std::cout << "    Family web: " << p.familyWeb << std::endl;
         std::cout << "    Works for: " << nameOf( people, p.employer ) << std::endl;
         std::cout << "    Employs:" << std::endl;

         for (
            vectorID::const_iterator itEmployee = p.employees.begin();
         	itEmployee != p.employees.end();
         	++itEmployee )
         {
            std::cout << "      " << nameOf( people, *itEmployee ) << std::endl;
         }
      }
   }
   catch (const XMLException& toCatch)
   {
      char* message = XMLString::transcode(toCatch.getMessage());
      std::cout << "Exception message is: \n"
           << message << "\n";
      XMLString::release(&message);
      return -1;
   }
   catch (const SAXParseException& toCatch)
   {
      char* message = XMLString::transcode(toCatch.getMessage());
      std::cout << "Exception message is: \n"
           << message << "\n";
      XMLString::release(&message);
      return -1;
   }
   catch ( const std::exception &x )
   {
      std::cout << x.what() << std::endl;
   }
	return 0;
}
