// Copyright (2003) Michael L Perry.
// Permission is granted to use this source code without restriction
// privided the copyright notice appears without modification.

// MemFuncPtr.h
//
// Defines a class template that generates facets for the XML interpreter.
// Uses pointers to member functions.

#include "XMLInterpreter.h"

#ifndef _MEMFUNCPTR_H
#define _MEMFUNCPTR_H

template< class IMPL >
class FXMLElementHandler : public IXMLElementHandler
{
public:
   // Define aliases for member function pointer types.
   typedef IXMLElementHandler *(IMPL::*pfnChild)(
      const XMLCh *const name );
   typedef void (IMPL::*pfnStart)(
      AttributeList &attrs );
   typedef void (IMPL::*pfnEnd)();
   typedef void (IMPL::*pfnCharacters)(
      const XMLCh *const chars,
      const unsigned int length);

   // The constructor takes a pointer to the implementation and pointers
   // to all member functions.
   FXMLElementHandler(
      IMPL *impl,
      pfnChild c,
      pfnStart s,
      pfnEnd e,
      pfnCharacters ch ) :

      m_impl( impl ),
      m_child( c ),
      m_start( s ),
      m_end( e ),
      m_characters( ch ) {}
            
   // Delegate to the given member functions.
   IXMLElementHandler *child(
      const XMLCh *const name )
   {
      return (m_impl->*m_child)(name);
   }

   void start(
      AttributeList &attrs )
   {
      (m_impl->*m_start)(attrs);
   }

   void end()
   {
      (m_impl->*m_end)();
   }

   void characters(
      const XMLCh *const chars,
      const unsigned int length)
   {
      (m_impl->*m_characters)(chars, length);
   }

private:
   IMPL *m_impl;
   pfnChild m_child;
   pfnStart m_start;
   pfnEnd m_end;
   pfnCharacters m_characters;
};

#endif
